"""Deprecated: Use BaseASLModel instead.

This module is kept for backward compatibility. New code should
import from ``osipy.asl.quantification.base``.
"""

from osipy.asl.quantification.base import BaseASLModel

# Backward compatibility alias
BaseATTModel = BaseASLModel
