"""Common model base classes."""

from osipy.common.models.base import BaseComponent, BaseSignalModel
from osipy.common.models.fittable import BaseBoundModel, FittableModel

__all__ = ["BaseBoundModel", "BaseComponent", "BaseSignalModel", "FittableModel"]
